@@ -1,4 +1,4 @@
-# Copyright (c) 2003-2010 LOGILAB S.A. (Paris, FRANCE).
+# Copyright (c) 2003-2012 LOGILAB S.A. (Paris, FRANCE).
 # Copyright (c) 2009-2010 Arista Networks, Inc.
 # http://www.logilab.fr/ -- mailto:contact@logilab.fr
 # This program is free software; you can redistribute it and/or modify it under
@@ -23,7 +23,12 @@ from logilab.astng import are_exclusive
 from pylint.interfaces import IASTNGChecker
 from pylint.reporters import diff_string
 from pylint.checkers import BaseChecker, EmptyReport
-from pylint.checkers.utils import check_messages, clobber_in_except, is_inside_except
+from pylint.checkers.utils import (
+    check_messages,
+    clobber_in_except,
+    is_inside_except,
+    safe_infer,
+    )
 
 
 import re
@@ -113,7 +118,7 @@ def redefined_by_decorator(node):
     if node.decorators:
         for decorator in node.decorators.nodes:
             if (isinstance(decorator, astng.Getattr) and
-                decorator.expr.name == node.name):
+                getattr(decorator.expr, 'name', None) == node.name):
                 return True
     return False
 
@@ -124,27 +129,35 @@ class _BasicChecker(BaseChecker):
 class BasicErrorChecker(_BasicChecker):
     msgs = {
     'E0100': ('__init__ method is a generator',
+              'init-is-generator',
               'Used when the special class method __init__ is turned into a '
               'generator by a yield in its body.'),
     'E0101': ('Explicit return in __init__',
+              'return-in-init',
               'Used when the special class method __init__ has an explicit \
               return value.'),
     'E0102': ('%s already defined line %s',
+              'function-redefined',
               'Used when a function / class / method is redefined.'),
     'E0103': ('%r not properly in loop',
+              'not-in-loop',
               'Used when break or continue keywords are used outside a loop.'),
 
     'E0104': ('Return outside function',
+              'return-outside-function',
               'Used when a "return" statement is found outside a function or '
               'method.'),
     'E0105': ('Yield outside function',
+              'yield-outside-function',
               'Used when a "yield" statement is found outside a function or '
               'method.'),
     'E0106': ('Return with argument inside generator',
+              'return-arg-in-generator',
               'Used when a "return" statement with an argument is found '
               'outside in a generator function or method (e.g. with some '
               '"yield" statements).'),
     'E0107': ("Use of the non-existent %s operator",
+              'nonexistent-operator',
               "Used when you attempt to use the C-style pre-increment or"
               "pre-decrement operator -- and ++, which doesn't exist in Python."),
     }
@@ -242,54 +255,67 @@ functions, methods
     name = 'basic'
     msgs = {
     'W0101': ('Unreachable code',
+              'unreachable',
               'Used when there is some code behind a "return" or "raise" \
               statement, which will never be accessed.'),
     'W0102': ('Dangerous default value %s as argument',
+              'dangerous-default-value',
               'Used when a mutable value as list or dictionary is detected in \
               a default value for an argument.'),
     'W0104': ('Statement seems to have no effect',
+              'pointless-statement',
               'Used when a statement doesn\'t have (or at least seems to) \
               any effect.'),
     'W0105': ('String statement has no effect',
+              'pointless-string-statement',
               'Used when a string is used as a statement (which of course \
               has no effect). This is a particular case of W0104 with its \
               own message so you can easily disable it if you\'re using \
               those strings as documentation, instead of comments.'),
     'W0106': ('Expression "%s" is assigned to nothing',
+              'expression-not-assigned',
               'Used when an expression that is not a function call is assigned\
               to nothing. Probably something else was intended.'),
     'W0108': ('Lambda may not be necessary',
+              'unnecessary-lambda',
               'Used when the body of a lambda expression is a function call \
               on the same argument list as the lambda itself; such lambda \
               expressions are in all but a few cases replaceable with the \
               function being called in the body of the lambda.'),
     'W0109': ("Duplicate key %r in dictionary",
+              'duplicate-key',
               "Used when a dictionary expression binds the same key multiple \
               times."),
     'W0122': ('Use of the exec statement',
+              'exec-statement',
               'Used when you use the "exec" statement, to discourage its \
               usage. That doesn\'t mean you can not use it !'),
 
     'W0141': ('Used builtin function %r',
+              'bad-builtin',
               'Used when a black listed builtin function is used (see the '
               'bad-function option). Usual black listed functions are the ones '
               'like map, or filter , where Python offers now some cleaner '
               'alternative like list comprehension.'),
     'W0142': ('Used * or ** magic',
+              'star-args',
               'Used when a function or method is called using `*args` or '
               '`**kwargs` to dispatch arguments. This doesn\'t improve '
               'readability and should be used with care.'),
     'W0150': ("%s statement in finally block may swallow exception",
+              'lost-exception',
               "Used when a break or a return statement is found inside the \
               finally clause of a try...finally block: the exceptions raised \
               in the try clause will be silently swallowed instead of being \
               re-raised."),
     'W0199': ('Assert called on a 2-uple. Did you mean \'assert x,y\'?',
+              'assert-on-tuple',
               'A call of assert on a tuple will always evaluate to true if '
               'the tuple is not empty, and will always evaluate to false if '
               'it is.'),
 
     'C0121': ('Missing required attribute "%s"', # W0103
+              'missing-module-attribute',
               'Used when an attribute required for modules is missing.'),
 
     }
@@ -428,6 +454,12 @@ functions, methods
                 else:
                     msg = '%s (%s)' % (default.as_string(), value.as_string())
                 self.add_message('W0102', node=node, args=(msg,))
+            if value.qname() == '__builtin__.set':
+                if isinstance(default, astng.CallFunc):
+                    msg = default.as_string()
+                else:
+                    msg = '%s (%s)' % (default.as_string(), value.qname())
+                self.add_message('W0102', node=node, args=(msg,))
 
     @check_messages('W0101', 'W0150')
     def visit_return(self, node):
@@ -502,13 +534,13 @@ functions, methods
         """check the use of an assert statement on a tuple."""
         if node.fail is None and isinstance(node.test, astng.Tuple) and \
            len(node.test.elts) == 2:
-             self.add_message('W0199', line=node.fromlineno, node=node)
+            self.add_message('W0199', line=node.fromlineno, node=node)
 
     @check_messages('W0109')
     def visit_dict(self, node):
         """check duplicate key in dictionary"""
         keys = set()
-        for k, v in node.items:
+        for k, _ in node.items:
             if isinstance(k, astng.Const):
                 key = k.value
                 if key in keys:
@@ -552,9 +584,11 @@ functions, methods
 class NameChecker(_BasicChecker):
     msgs = {
     'C0102': ('Black listed name "%s"',
+              'blacklisted-name',
               'Used when the name is listed in the black list (unauthorized \
               names).'),
-    'C0103': ('Invalid name "%s" (should match %s)',
+    'C0103': ('Invalid name "%s" for type %s (should match %s)',
+              'invalid-name',
               'Used when the name doesn\'t match the regular expression \
               associated to its type (constant, variable, class...).'),
 
@@ -644,7 +678,7 @@ class NameChecker(_BasicChecker):
     @check_messages('C0102', 'C0103')
     def visit_class(self, node):
         self._check_name('class', node.name, node)
-        for attr, anodes in node.instance_attrs.items():
+        for attr, anodes in node.instance_attrs.iteritems():
             self._check_name('attr', attr, anodes[0])
 
     @check_messages('C0102', 'C0103')
@@ -687,7 +721,6 @@ class NameChecker(_BasicChecker):
             clobbering, _ = clobber_in_except(node)
             if clobbering:
                 return
-
         if name in self.config.good_names:
             return
         if name in self.config.bad_names:
@@ -696,17 +729,23 @@ class NameChecker(_BasicChecker):
             return
         regexp = getattr(self.config, node_type + '_rgx')
         if regexp.match(name) is None:
-            self.add_message('C0103', node=node, args=(name, regexp.pattern))
+            type_label = {'inlinedvar': 'inlined variable',
+                          'const': 'constant',
+                          'attr': 'attribute',
+                          }.get(node_type, node_type)
+            self.add_message('C0103', node=node, args=(name, type_label, regexp.pattern))
             self.stats['badname_' + node_type] += 1
 
 
 class DocStringChecker(_BasicChecker):
     msgs = {
     'C0111': ('Missing docstring', # W0131
+              'missing-docstring',
               'Used when a module, function, class or method has no docstring.\
               Some special methods like __init__ doesn\'t necessary require a \
               docstring.'),
     'C0112': ('Empty docstring', # W0132
+              'empty-docstring',
               'Used when a module, function, class or method has an empty \
               docstring (it would be too easy ;).'),
     }
@@ -762,8 +801,9 @@ class DocStringChecker(_BasicChecker):
 class PassChecker(_BasicChecker):
     """check is the pass statement is really necessary"""
     msgs = {'W0107': ('Unnecessary pass statement',
+                      'unnecessary-pass',
                       'Used when a "pass" statement that can be avoided is '
-                      'encountered.)'),
+                      'encountered.'),
             }
 
     def visit_pass(self, node):
@@ -771,6 +811,36 @@ class PassChecker(_BasicChecker):
             self.add_message('W0107', node=node)
 
 
+class LambdaForComprehensionChecker(_BasicChecker):
+    """check for using a lambda where a comprehension would do.
+
+    See <http://www.artima.com/weblogs/viewpost.jsp?thread=98196>
+    where GvR says comprehensions would be clearer.
+    """
+
+    msgs = {'W0110': ('map/filter on lambda could be replaced by comprehension',
+                      'deprecated-lambda',
+                      'Used when a lambda is the first argument to "map" or '
+                      '"filter". It could be clearer as a list '
+                      'comprehension or generator expression.'),
+            }
+
+    @check_messages('W0110')
+    def visit_callfunc(self, node):
+        """visit a CallFunc node, check if map or filter are called with a
+        lambda
+        """
+        if not node.args:
+            return
+        if not isinstance(node.args[0], astng.Lambda):
+            return
+        infered = safe_infer(node.func)
+        if (infered
+            and infered.parent.name == '__builtin__'
+            and infered.name in ['map', 'filter']):
+            self.add_message('W0110', node=node)
+
+
 def register(linter):
     """required method to auto register this checker"""
     linter.register_checker(BasicErrorChecker(linter))
@@ -778,3 +848,4 @@ def register(linter):
     linter.register_checker(NameChecker(linter))
     linter.register_checker(DocStringChecker(linter))
     linter.register_checker(PassChecker(linter))
+    linter.register_checker(LambdaForComprehensionChecker(linter))
