@@ -1,4 +1,4 @@
-# Copyright (c) 2003-2012 LOGILAB S.A. (Paris, FRANCE).
+# Copyright (c) 2003-2013 LOGILAB S.A. (Paris, FRANCE).
 # http://www.logilab.fr/ -- mailto:contact@logilab.fr
 #
 # This program is free software; you can redistribute it and/or modify it under
@@ -63,6 +63,9 @@ MSGS = {
     'E0603': ('Undefined variable name %r in __all__',
               'undefined-all-variable',
               'Used when an undefined variable name is referenced in __all__.'),
+    'E0604': ('Invalid object %r in __all__, must contain only strings',
+              'invalid-all-object',
+              'Used when an invalid (non-string) object occurs in __all__.'),
     'E0611': ('No name %r in module %r',
               'no-name-in-module',
               'Used when a name cannot be found in a module.'),
@@ -172,14 +175,23 @@ builtins. Remember that you should avoid to define new builtins when possible.'
         # attempt to check for __all__ if defined
         if '__all__' in node.locals:
             assigned = node.igetattr('__all__').next()
-            for elt in getattr(assigned, 'elts', ()):
-                elt_name = elt.value
-                # If elt is in not_consumed, remove it from not_consumed
-                if elt_name in not_consumed:
-                    del not_consumed[elt_name]
-                    continue
-                if elt_name not in node.locals:
-                    self.add_message('E0603', args=elt_name, node=elt)
+            if assigned is not astng.YES:
+                for elt in getattr(assigned, 'elts', ()):
+                    try:
+                        elt_name = elt.infer().next()
+                    except astng.InferenceError:
+                        continue
+
+                    if not isinstance(elt_name, astng.Const) or not isinstance(elt_name.value, basestring):
+                        self.add_message('E0604', args=elt.as_string(), node=elt)
+                        continue
+                    elt_name = elt.value
+                    # If elt is in not_consumed, remove it from not_consumed
+                    if elt_name in not_consumed:
+                        del not_consumed[elt_name]
+                        continue
+                    if elt_name not in node.locals:
+                        self.add_message('E0603', args=elt_name, node=elt)
         # don't check unused imports in __init__ files
         if not self.config.init_import and node.package:
             return
@@ -260,7 +272,7 @@ builtins. Remember that you should avoid to define new builtins when possible.'
             if is_inside_except(stmt):
                 continue
             if name in globs and not isinstance(stmt, astng.Global):
-                line = globs[name][0].lineno
+                line = globs[name][0].fromlineno
                 self.add_message('W0621', args=(name, line), node=stmt)
             elif is_builtin(name):
                 # do not print Redefining builtin for additional builtins
